#!/usr/bin/env python
from distutils.core import setup

setup(
    name='chunker',
    version='1.1',
    description='Easy Chunk-Based File Structure Parser',
    author='Archangel_SDY',
    author_email='Archangel.SDY@gmail.com',
    url='http://archangelsdy.github.io/chunker/',
    packages=['chunker']
)
